package com.persagy.proxy.adm.controller;

import cn.hutool.core.collection.CollUtil;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmShaft;
import com.persagy.proxy.adm.request.AdmCreateRequest;
import com.persagy.proxy.adm.request.AdmQueryCriteria;
import com.persagy.proxy.adm.request.AdmResponse;
import com.persagy.proxy.adm.service.IAdmShaftService;
import com.persagy.proxy.common.entity.InstanceUrlParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;
import java.util.List;

/**
 * Adm 竖井 API
 * @author lvxy
 * @date 2021/8/16
 */

@RestController
@RequestMapping("/object/shaft")
public class AdmShaftController {
    @Autowired
    private IAdmShaftService service;
    @Value("${middleware.group.code}")
    private String groupCode;


    /**
     * 统计
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/count")
    public AdmResponse count(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        request.setOnlyCount(true);
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 创建
     * @param createRequest
     * @return
     * @throws Exception
     */
    @PostMapping("/create")
    public AdmResponse create(@RequestBody AdmCreateRequest<AdmShaft> createRequest, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmShaft> vos = CollUtil.newArrayList(createRequest.getContent());
        vos = service.doInsert(context, AdmShaft.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 根据ID删除
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/delete")
    public AdmResponse delete(@QueryParam("projectId") String projectId, @RequestBody List<AdmShaft> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        service.doDelete(context, vos);
        return AdmResponse.success();
    }

    /**
     * 更新
     * @param createRequest
     * @return
     * @throws Exception
     */
    @PostMapping("/update")
    public AdmResponse update(@RequestBody AdmCreateRequest<AdmShaft> createRequest, @QueryParam("projectId") String projectId) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        List<AdmShaft> vos = CollUtil.newArrayList(createRequest.getContent());
        vos = service.doUpdate(context, AdmShaft.class, vos);
        return AdmResponse.success(vos);
    }

    /**
     * 查询
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public AdmResponse query(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询项目下已有的竖井类型
     * 此方法可能会弃用
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query-shaft-functype")
    public AdmResponse queryShFuncType(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询竖井信息-包含统计信息
     * 此方法可能会弃用
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query-with-count")
    public AdmResponse queryWithCount(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询关联元空间
     * 可能移至关系
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/relate-space")
    public AdmResponse queryRelateSp(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询和当前竖井有关联的业务空间
     * shaftId 为竖井id
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/shaft-zoneSpaceBaseTable")
    public AdmResponse queryZoneSp(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询和当前竖井有关联的业务空间所在建筑及楼层信息
     * shaftId 为竖井id
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/shaft-zoneSpaceBaseTable-building")
    public AdmResponse queryZoneSpBd(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }

    /**
     * 查询和当前竖井没有关联的业务空间
     * shaftId 为竖井id
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/un-shaft-zoneSpaceBaseTable")
    public AdmResponse queryUnZoneSp(@QueryParam("projectId") String projectId,
                             @RequestBody AdmQueryCriteria request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, AdmCommonConstant.APP_ID);
        request.setName("shaft");
        return service.doQuery(context, request, AdmShaft.class);
    }
}
