package com.persagy.proxy.adm.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.google.common.collect.Lists;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.dmp.digital.client.DigitalRelationFacade;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.service.IReportDownloadService;
import com.persagy.proxy.adm.strategy.QueryRelationObjectContext;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 报表/模板下载
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年8月31日 下午4:43:15
 */
@Slf4j
@Service
public class ReportDownloadService implements IReportDownloadService {

	public static final String cADID = "cADID";
	
	public static final String pe2bd = "pe2bd";
	public static final String pe2fl = "pe2fl";

	@Value("${middleware.group.code}")
	private String defaultCode;

	@Autowired
	private QueryRelationObjectContext queryRelationObjectContext;
	
	@Override
	public ObjectNode getObjectNode(String groupCode, String projectId, String id) {
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode objectNode = objectMapper.createObjectNode();
		objectNode.put("id", id);
		queryCriteria.setCriteria(objectNode);
		List<ObjectNode> objectNodes = DigitalObjectFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return CollectionUtil.isEmpty(objectNodes) ? objectMapper.createObjectNode() : objectNodes.get(0); 
	}
	
	@Override
	public ObjectNode getObjectByCodeAndId(String groupCode, String projectId, String graphCode, String relCode, String objFrom, String objTo) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode objectNode = objectMapper.createObjectNode();
		objectNode.put("graphCode", graphCode);
		objectNode.put("relCode", relCode);
		objectNode.put("objFrom", objFrom);
		objectNode.put("objTo", objTo);
		queryCriteria.setCriteria(objectNode);
		List<ObjectRelation> objectRelations = DigitalRelationFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectRelations)) {
			return objectMapper.createObjectNode();
		}
		
		// 获取关联对象的本地名称
		ObjectRelation objectRelation = objectRelations.get(0);
		String relationId = StrUtil.isBlank(objFrom) ? objectRelation.getObjFrom() : objectRelation.getObjTo();
		return this.getObjectNode(groupCode, projectId, relationId); 
	}
	
	@Override
	public List<ObjectRelation> findObjectRelationList(String groupCode, String projectId, String graphCode, String relCode) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		
		// 获取出关联对象
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		criteria.put("graphCode", graphCode);
		criteria.put("relCode", relCode);
		queryCriteria.setCriteria(criteria);
		List<ObjectRelation> objectRelations = DigitalRelationFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		return CollectionUtil.isEmpty(objectRelations) ? Lists.newArrayList() : objectRelations;
	}
	
	
	@Override
	public List<AdmRelationObject> reportRelationObject(String groupCode, String projectId, String relType, String code, String zoneType) {
		groupCode = StrUtil.isBlank(groupCode) ? defaultCode : groupCode;
		log.info("下载报表： groupCode[{}], projectId[{}], relType[{}], code[{}]", groupCode, projectId, relType, code);
		return this.queryRelationObjectContext.findRelationObject(groupCode, projectId, relType);
	}
	
}
