package com.persagy.proxy.adm.strategy;

import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Component;

import com.google.common.collect.Lists;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.strategy.relationdata.QueryRelationObjectStrategy;

/**
 * 获取关系对象的上下文类
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年9月2日 下午10:44:32
 */
@Component
public class QueryRelationObjectContext {

	/**
	 * 策略执行类
	 */
	private Map<String, QueryRelationObjectStrategy> relationObjectStrategyMap;
	
	/**
	 * 自动注入 byName
	 * 
	 * @param relationObjectStrategyMap
	 */
	@Resource
	public void relationObjectStrategyMap(Map<String, QueryRelationObjectStrategy> relationObjectStrategyMap) {
		this.relationObjectStrategyMap = relationObjectStrategyMap;
	}
	
	/**
	 * 获取关系对象数据
	 * 
	 * @param groupCode
	 * @param projectId
	 * @param relType
	 * @return
	 */
	public List<AdmRelationObject> findRelationObject(String groupCode, String projectId, String relType) {
		QueryRelationObjectStrategy strategy = this.relationObjectStrategyMap.get(relType);
		return strategy == null ? Lists.newArrayList() : strategy.findRelationObject(groupCode, projectId, relType);
	}
	
}
