package com.persagy.proxy.adm.strategy.relationdata;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.digital.client.DigitalObjectFacade;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.service.IReportDownloadService;

import cn.hutool.core.collection.CollectionUtil;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年9月2日 下午11:00:26
 */
public abstract class AbstractQueryRelationObject implements QueryRelationObjectStrategy {

	protected String groupCode;
	
	protected String projectId;
	
	protected IReportDownloadService reportDownloadService;
	
	public AbstractQueryRelationObject(IReportDownloadService reportDownloadService) {
		this.reportDownloadService = reportDownloadService;
	}
	
	@Override
	public List<AdmRelationObject> findRelationObject(String groupCode, String projectId, String relType) {
		this.groupCode = groupCode;
		this.projectId = projectId;
		return this.findRelationObject(relType);
	}
	
	/**
	 * 查询出指定设备信息
	 * 
	 * @param relType
	 * @return 返回ADM所需要的数据
	 */
	protected abstract List<AdmRelationObject> findRelationObject(String relType);
	
	/**
	 * 根据项目ID，查询出所有的对象信息
	 * 
	 * @param requestData
	 * @param relType
	 * @param extraMapping
	 * @return
	 */
	protected List<AdmRelationObject> queryAllObjectInfo(QueryCriteria queryCriteria, String relType) {
		List<ObjectNode> tempList = new ArrayList<ObjectNode>();
		
		// 获取数据
		this.queryPageObjectInfo(tempList, queryCriteria, queryCriteria.getPage());
		
		// 转换中台数据
		return this.handleObjectDigital(tempList, relType);
	}
	
	/**
	 * 分页查询
	 * 
	 * @param tempList
	 * @param queryCriteria
	 * @param groupCode
	 * @param projectId
	 * @param page
	 * @param pageSize
	 */
	protected void queryPageObjectInfo(List<ObjectNode> tempList, QueryCriteria queryCriteria, Long page) {
		queryCriteria.setPage(page);
		List<ObjectNode> objectNodes = DigitalObjectFacade.query(groupCode, projectId, AdmCommonConstant.APP_ID, AdmCommonConstant.USER_ID, queryCriteria);
		if (CollectionUtil.isEmpty(objectNodes)) {
			return;
		}
		tempList.addAll(objectNodes);
		this.queryPageObjectInfo(tempList, queryCriteria, ++page);
	}
	
	/**
	 * 处理中台响应数据，转换为ADM所需要的数据
	 * 
	 * @param tempList
	 * @param groupCode
	 * @param projectId
	 * @param relType
	 * @return
	 */
	private List<AdmRelationObject> handleObjectDigital(List<ObjectNode> tempList, String relType) {
		// 第一次遍历
		// id -> object
		Map<String, ObjectNode> tempAllMap = new HashMap<String, ObjectNode>(tempList.size());
		for (ObjectNode objectNode : tempList) {
			String id = objectNode.get("id").asText();
			tempAllMap.put(id, objectNode);
		}
		
		return this.handleObjectDigital(tempAllMap, relType);
	}
	
	
	/**
	 * 处理中台响应数据，转换为ADM所需要的数据
	 * 
	 * @param tempAllMap 对象ID -> 对象
	 * @param relType
	 * @return 不允许返回null
	 */
	protected abstract List<AdmRelationObject> handleObjectDigital(Map<String, ObjectNode> tempAllMap, String relType);	
	
	/**
	 * 封装响应数据，具体的传值，请仔细侦查，这里仅返回共有字段，特殊字段，自行赋值
	 * 
	 * @param master
	 * @param masterType
	 * @param slave
	 * @param slaveType
	 * @return
	 */
	protected AdmRelationObject convertObject(ObjectNode master, String masterType, ObjectNode slave, String slaveType) {
		AdmRelationObject relationObject = new AdmRelationObject();
		
		// ID
		String masterId = master.get("id") == null ? AdmCommonConstant.EMPTY  : master.get("id").asText();
		relationObject.setMasterCode(masterId);
		relationObject.setMasterId(masterId);
		// 图纸编码
		relationObject.setMasterCadId(master.get("cADID") == null ? AdmCommonConstant.EMPTY  : master.get("cADID").asText());
		// 名称
		relationObject.setMasterName(master.get("name") == null ? AdmCommonConstant.EMPTY  : master.get("name").asText());
		// 本地ID
		relationObject.setMasterLocalId(master.get("localId") == null ? AdmCommonConstant.EMPTY  : master.get("localId").asText());
		// 本地名称
		relationObject.setMasterLocalName(master.get("localName") == null ? AdmCommonConstant.EMPTY  : master.get("localName").asText());
		// 类型
		relationObject.setMasterType(masterType);
		
		// ID
		String slaveId = slave.get("id") == null ? AdmCommonConstant.EMPTY  : slave.get("id").asText();
		relationObject.setSlaveCode(slaveId);
		relationObject.setSlaveId(slaveId);
		// 图纸编码
		relationObject.setSlaveCadId(master.get("cADID") == null ? AdmCommonConstant.EMPTY  : master.get("cADID").asText());
		// 名称
		relationObject.setSlaveName(slave.get("name") == null ? AdmCommonConstant.EMPTY  : slave.get("name").asText());
		// 本地ID
		relationObject.setSlaveLocalId(slave.get("localId") == null ? AdmCommonConstant.EMPTY  : slave.get("localId").asText());
		// 本地名称
		relationObject.setSlaveLocalName(slave.get("localName") == null ? AdmCommonConstant.EMPTY  : slave.get("localName").asText());
		// 类型
		relationObject.setSlaveType(slaveType);
		
		return relationObject;
	}
	
}
