package com.persagy.proxy.adm.strategy.relationdata;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmObjectType;
import com.persagy.proxy.adm.constant.AdmRelationType;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.constant.RelCodeEnum;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.service.IReportDownloadService;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年9月3日 上午9:42:46
 */
@Component(value = AdmRelationType.SH2FL_ARCHFORARCH)
public class Sh2FlForRelationObject extends AbstractQueryRelationObject {

	public Sh2FlForRelationObject(IReportDownloadService reportDownloadService) {
		super(reportDownloadService);
	}

	@Override
	protected List<AdmRelationObject> findRelationObject(String relType) {
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		// 参数条件
		ObjectNode objType = objectMapper.createObjectNode();
		ArrayNode objTypes = objType.putArray(WHERE_IN);
		objTypes.add(AdmObjectType.SHAFT.getIndex()).add(AdmObjectType.FLOOR.getIndex());
		
		criteria.set(OBJ_TYPE, objType);
		queryCriteria.setCriteria(criteria);
		queryCriteria.setPage(1L);
		queryCriteria.setSize(1000L);
		
		return this.queryAllObjectInfo(queryCriteria, relType);
	}

	@Override
	protected List<AdmRelationObject> handleObjectDigital(Map<String, ObjectNode> tempAllMap, String relType) {
		Set<String> allKeys = tempAllMap.keySet();
		List<AdmRelationObject> resultList = new ArrayList<AdmRelationObject>();
		
		List<ObjectRelation> relationList = this.reportDownloadService.findObjectRelationList(groupCode, projectId, 
				AdmRelationTypeEnum.SH2FL_ARCHFORARCH.getGraphCode(), AdmRelationTypeEnum.SH2FL_ARCHFORARCH.getRelCode());
		for (ObjectRelation objectRelation : relationList) {
			String shaftId = objectRelation.getObjFrom();
			String floorId = objectRelation.getObjTo();
			if (!allKeys.contains(shaftId) || !allKeys.contains(floorId)) {
				continue;
			}
			ObjectNode shaft = tempAllMap.get(shaftId);
			ObjectNode floor = tempAllMap.get(floorId);
			// 获取建筑的信息
			ObjectNode buildingInfo = this.reportDownloadService.getObjectByCodeAndId(groupCode, projectId, AdmRelationTypeEnum.SH2FL_ARCHFORARCH.getGraphCode(), 
					RelCodeEnum.Sh2Bd.name(), shaftId, null);

			AdmRelationObject convertObject = this.convertObject(shaft, "竖井", floor, "楼层");
			String buildingName = buildingInfo.get("localName") == null ? AdmCommonConstant.EMPTY : buildingInfo.get("localName").asText();
			convertObject.setSlaveBuildingName(buildingName);
			convertObject.setSlaveBelong(buildingName);
			resultList.add(convertObject);
		}
		
		return resultList;
	}

}
