package com.persagy.proxy.adm.strategy.relationdata;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.digital.entity.ObjectRelation;
import com.persagy.proxy.adm.constant.AdmCommonConstant;
import com.persagy.proxy.adm.constant.AdmObjectType;
import com.persagy.proxy.adm.constant.AdmRelationType;
import com.persagy.proxy.adm.constant.AdmRelationTypeEnum;
import com.persagy.proxy.adm.constant.RelCodeEnum;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.service.IReportDownloadService;

import cn.hutool.core.util.StrUtil;

/**
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021年9月3日 上午9:42:46
 */
@Component(value = AdmRelationType.SY2SP_FOR)
public class Sy2SpForRelationObject extends AbstractQueryRelationObject {

	public Sy2SpForRelationObject(IReportDownloadService reportDownloadService) {
		super(reportDownloadService);
	}

	@Override
	protected List<AdmRelationObject> findRelationObject(String relType) {
		ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
		QueryCriteria queryCriteria = new QueryCriteria();
		ObjectNode criteria = objectMapper.createObjectNode();
		// 参数条件
		ObjectNode objType = objectMapper.createObjectNode();
		ArrayNode objTypes = objType.putArray(WHERE_IN);
		objTypes.add(AdmObjectType.SYSTEM.getIndex()).add(AdmObjectType.SPACE.getIndex());
		
		criteria.set(OBJ_TYPE, objType);
		queryCriteria.setCriteria(criteria);
		queryCriteria.setPage(1L);
		queryCriteria.setSize(1000L);
		
		return this.queryAllObjectInfo(queryCriteria, relType);
	}

	@Override
	protected List<AdmRelationObject> handleObjectDigital(Map<String, ObjectNode> tempAllMap, String relType) {
		Set<String> allKeys = tempAllMap.keySet();
		List<AdmRelationObject> resultList = new ArrayList<AdmRelationObject>();
		
		List<ObjectRelation> relationList = this.reportDownloadService.findObjectRelationList(groupCode, projectId, 
				AdmRelationTypeEnum.SY2SP_FOR.getGraphCode(), AdmRelationTypeEnum.SY2SP_FOR.getRelCode());
		for (ObjectRelation objectRelation : relationList) {
			String systemId = objectRelation.getObjFrom();
			String spaceId = objectRelation.getObjTo();
			if (!allKeys.contains(systemId) || !allKeys.contains(spaceId)) {
				continue;
			}
			ObjectNode system = tempAllMap.get(systemId);
			ObjectNode space = tempAllMap.get(spaceId);
			
			// 获取楼层信息
			ObjectNode floorInfo = this.reportDownloadService.getObjectByCodeAndId(groupCode, projectId, AdmRelationTypeEnum.SY2SP_FOR.getGraphCode(), 
					RelCodeEnum.Sy2Bd.name(), systemId, null);
			ObjectNode buildingInfo = this.reportDownloadService.getObjectByCodeAndId(groupCode, projectId, AdmRelationTypeEnum.SY2SP_FOR.getGraphCode(), 
					RelCodeEnum.Sy2Fl.name(), systemId, null);
			
			AdmRelationObject convertObject = this.convertObject(system, "系统", space, "空间");
			String buildingName = buildingInfo.get("localName") == null ? AdmCommonConstant.EMPTY : buildingInfo.get("localName").asText();
			String floorName = floorInfo.get("localName") == null ? AdmCommonConstant.EMPTY : floorInfo.get("localName").asText();
			convertObject.setSlaveBuildingName(buildingName);
			convertObject.setSlaveFloorName(floorName);
			if (StrUtil.isBlank(floorName)) {
				convertObject.setSlaveBelong(buildingName);
			} else {
				convertObject.setSlaveBelong((StrUtil.isBlank(buildingName) ? AdmCommonConstant.EMPTY 
						: StrUtil.join(AdmCommonConstant.LINE_THROUGH, buildingName, floorName)));
			}
			
			resultList.add(convertObject);
		}
		
		return resultList;
	}

}
